package controllers;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;

import models.file.ImageDirectory;
import models.util.image.DataPie;
import models.util.image.FusionMgr;
import play.Logger;
import play.i18n.Messages;
import play.mvc.Controller;
import play.mvc.results.NotModified;
import util.ConfigReader;

/**
 * Speedysprite web service class.
 * 
 * @author NorthPoint
 */
public class Application extends Controller {

    /**
     * Generates the dynamic CSS file containing image references and 
     * associated sprite file coordinates. If image file caching is enabled
     * this method also generates the spite image and saves the image to disk.
     * 
     * @param dirPath
     * @param paths
     * @throws FileNotFoundException
     */
    public static void css(String paths) throws FileNotFoundException {
    	DataPie dataPie;
    	String spriteUri;
    	List<File> imageFiles;
    	FusionMgr fusionMgr = new FusionMgr();
    	
    	/* Invoked with a list of image paths */
    	if (paths != null) {
        	imageFiles = ImageDirectory.getImagesFromPathList(paths);
    		spriteUri = "sprite/" + paths;
    	/* Invoked with a directory path */
    	} else {
    		dataPie = null;
    		spriteUri = null;
    		imageFiles = null;
    		Logger.warn(Messages.get("error.invalid_css_invocation"));
    		return;
    	}

		handleClientCaching(imageFiles);
    	//pass in null to avoid writing out actual sprite img
		dataPie = fusionMgr.fuse(imageFiles, (OutputStream)null);

    	Date now = new Date();
    	int sliceCount = dataPie.getSlices().size();
    	
    	renderArgs.put("header", Messages.get("message.css_header", now, sliceCount));
		renderArgs.put("dataPie", dataPie);
		renderArgs.put("sprite", spriteUri);
		renderArgs.put("now", new Date());
		render("Application/sprite.css");
    }

    /**
     * Generates or serves the generated sprite image file.
     * 
     * @param dirPath
     * @param paths
     * @throws FileNotFoundException
     */
    public static void sprite(String paths) throws FileNotFoundException {
    	FusionMgr fusionMgr = new FusionMgr();
    	List<File> imageFiles;
    	
    	/* Invoked with a list of image paths */
    	if (paths != null) {
    		imageFiles = ImageDirectory.getImagesFromPathList(paths);
    	/* Invoked with a directory path */
    	} else {
    		imageFiles = null;
    	}

    	long mostRecentTimestamp = handleClientCaching(imageFiles);    	
    	response.setContentTypeIfNotSet("image/" + FusionMgr.OUTPUT_FILE_EXT);
    	
    	/* Determine whether to 'cache' the sprite in a file or stream binary */
    	if (ConfigReader.spriteCacheDir.value() != null) {
    		File spriteFile = fusionMgr.generatedHashFile(imageFiles);
    		
    		if(spriteFile.exists() && spriteFile.lastModified() > mostRecentTimestamp) {    			
    		} else {    		
    			fusionMgr.fuse(imageFiles, spriteFile);
    		}
    		
    		renderBinary(spriteFile);
    	} else {
    		fusionMgr.fuse(imageFiles, response.out);
    	}
    }
    
    /**
     * Establishes response caching headers based on latest file last modified date.
     * 
     * @param pathList
     * @throws FileNotFoundException
     */
    private static long handleClientCaching(List<File> imageFiles) throws FileNotFoundException {
    	//if(true) return new Date().getTime(); //handy for debugging

		File mrf = ImageDirectory.getMostRecent(imageFiles);
		long lastModified = mrf.lastModified();
		
		String etag = FusionMgr.hashName(imageFiles);
		if (!request.isModified(etag, lastModified)) {
			throw new NotModified();
		}
		response.cacheFor(etag, ConfigReader.browserCacheSeconds + "s", lastModified);
		return lastModified;
	}
}
