package models.file;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import util.ConfigReader;

import models.util.file.FileUtil;

/**
 * Represents a single image directory with its immediate image children.
 * 
 * @author NorthPoint
 *
 */
public class ImageDirectory {

	public String path;
	public List<File> files;
	
	/**
	 * Full constructor
	 * @param path
	 */
	public ImageDirectory(String path) {
		this.path = path;
		files = new ArrayList<File>();
	}

	/**
	 * Adds a file to the directory.
	 * 
	 * @param file
	 */
	public void addFile(File file) {
		files.add(file);
	}
	
    /**
     * Provides a list of the image files contained in a directory.
     * 
     * @param root
     * @return A list of the images contained in the directory
     * @throws FileNotFoundException
     */
    public static List<ImageDirectory> getImages(File root) 
    		throws FileNotFoundException {
		FileUtil.validateDirectory(root);
		FileTreeNode rootNode = getFileTree(root, null);
		List<ImageDirectory> dirs = new ArrayList<ImageDirectory>();
		getDirectory(rootNode, null, dirs);
		return dirs;
    }

    /**
     * Retrieves all image files in a directory.
     * 
     * @param path
     * @return A list of the images contained in the directory
     * @throws FileNotFoundException
     */
    public static List<File> getImagesInDir(String path) throws FileNotFoundException {
    	File dir = new File(path);
		FileUtil.validateDirectory(dir);
		File[] fileArr = dir.listFiles(FileFilterType.IMAGES); 
		List<File> answer = Arrays.asList(fileArr);
		return answer;
	}
    
    /**
     * Retrieves all image files from a list of directory paths.
     * 
     * @param pathList
     * @return A list of the images contained in all specified directory paths
     * @throws FileNotFoundException
     */
	public static List<File> getImagesFromPathList(String pathList) throws FileNotFoundException {

		String[] paths = pathList.split(",");		
		List<File> imageFiles = new ArrayList<File>(paths.length);
	  	for (String path : paths) {
	  		path = ConfigReader.baseImagePath +"/" + path.trim();
	  		if(path.endsWith("/")) {
	  			imageFiles.addAll(getImagesInDir(path));
	  		} else {
	  			imageFiles.add(new File(path));
	  		}
	  	}
	  	
		return imageFiles;
	}

	//TODO: figure out where these paths are relative to
	public static String getPathListFromImages(List<File> images) {

		StringBuilder answer = new StringBuilder();

	  	for (Iterator<File> it=images.iterator(); it.hasNext(); ) {
			File f = it.next();
	  		answer.append(f.getName());
	  		if(it.hasNext()) answer.append(",");
	  	}
		return answer.toString();
	}
    
    /**
     * Recursively flatten a directory tree structure into a list of all directory nodes
     * and within each directory node a list of it's immediate child files.
     * @param currentNode
     * @param currentDir
     * @param dirs
     */
    private static void getDirectory(FileTreeNode currentNode, 
    		ImageDirectory currentDir, List<ImageDirectory> dirs) {
    	if (currentNode.hasChildren()) {
    		currentDir = new ImageDirectory(currentNode.file.getPath());
			dirs.add(currentDir);
		} else if (!currentNode.isDirectory()) {
			currentDir.addFile(currentNode.file);
		}
    	for (FileTreeNode node : currentNode.children) {
    		getDirectory(node, currentDir, dirs);
    	}
    }
	
    /**
     * Recursively build a tree representation of the images file structure.
     * 
     * @param start
     * @param parent
     * @return The current tree structure
     * @throws FileNotFoundException
     */
	private static FileTreeNode getFileTree(File start, FileTreeNode parent) 
			throws FileNotFoundException {
		File[] files = start.listFiles(FileFilterType.IMAGES);
		FileTreeNode startNode = new FileTreeNode(start, parent);
		List<File> filesDirs = Arrays.asList(files);
		for (File file : filesDirs) {
			if (file.isFile()) {
				startNode.addChild(new FileTreeNode(file, startNode));
			} else {
				startNode.addChild(getFileTree(file, parent));
			}
		}
		return startNode;
	}

	/**
	 * Retrieves the file with the latest modified date from a list of files.
	 * 
	 * @param files
	 * @return The file with the most recent modified date
	 */
	public static File getMostRecent(List<File> files) {
		long lastMod = 0;
		File lastFile = null;

		for (File f:files) {
			long mod = f.lastModified();
			if (mod > lastMod) {
				lastFile = f;
				lastMod = mod;
			}
		}
		return lastFile;
	}
}
